﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Traversal.Model
{
    /// <summary>
    /// Szintfolytonos bejárás típusa.
    /// </summary>
    public class LevelOrderTraversal : TreeTraversal // a fa bejárás absztakt osztálynak egy leszármazottja
    {
        private Queue<TreeNode> _LevelQueue; // a szintfolytonos bejáráshoz sor kell

        public override Boolean Running
        {
            get { return (_LevelQueue.Count > 0); } // akkor fut, ha van elem a sorban
        }

        public LevelOrderTraversal(BinarySearchTree tree)
            : base(tree) // meghívjuk az ős konstruktorát
        {
            _LevelQueue = new Queue<TreeNode>(); // létrehozunk egy új sort
       }

        public override void Start()
        {
            _LevelQueue.Clear(); // kiürítjük a sort, hátha volt benne elem
            if (_Tree.Root != null) // ha a fa nem üres
                _LevelQueue.Enqueue(_Tree.Root); // betesszük a gyökér elemet a sorba
        }

        public override void Step()
        {
            TreeNode node = _LevelQueue.Dequeue(); // kivesszük az első elemet
            if (node.Left != null) // ha a bal gyerek nem nulla
                _LevelQueue.Enqueue(node.Left); // felvesszük a sorba
            if (node.Right != null)
                _LevelQueue.Enqueue(node.Right);
            _VisitedNodes.Add(node); // hozzávesszük a bejárt csúcsok közé
        }

        public override void Draw(Graphics gr, Int32 width)
        {
            if (_LevelQueue.Count > 0 || _VisitedNodes.Count > 0) // ha már dolgozott az eljárás
                DrawNode(_Tree.Root, gr, 0, width, 20); // a saját rajzolófüggvényeket hívjuk
            else
                _Tree.Draw(gr, width); // különben a fát rajzoljuk ki
        }

        private void DrawNode(TreeNode node, Graphics gr, Int32 start, Int32 width, Int32 height)
        {
            Font font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold);
            Pen greenPen = new Pen(Color.Green); 
            Pen redPen = new Pen(Color.Red); 
            Pen blackPen = new Pen(Color.Black);
            SizeF keySize = gr.MeasureString(node.Key.ToString(), font);

            if (node.Left != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + width / 4, height + 50);
                DrawNode(node.Left, gr, start, width / 2, height + 50);
            }
            if (node.Right != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + 3 * width / 4, height + 50);
                DrawNode(node.Right, gr, start + width / 2, width / 2, height + 50);
            }

            gr.FillEllipse(Brushes.White, start + width / 2 - 15, height - 15, 30, 30); 
            if (_LevelQueue.Contains(node))
                gr.DrawEllipse(redPen, start + width / 2 - 15, height - 15, 30, 30);
            else if (_VisitedNodes.Contains(node))
                gr.DrawEllipse(greenPen, start + width / 2 - 15, height - 15, 30, 30);
            else
                gr.DrawEllipse(blackPen, start + width / 2 - 15, height - 15, 30, 30);
            gr.DrawString(node.Key.ToString(), font, Brushes.Black, start + width / 2 - keySize.Width / 2, height - keySize.Height / 2); // belerajzoljuk a kulcsot
        }
    }
}
