﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Traversal.Model
{
    /// <summary>
    /// Postorder bejárás típusa.
    /// </summary>
    public class PostorderTraversal : TreeTraversal // a fa bejárás absztakt osztálynak egy leszármazottja
    {
        private Stack<TreeNode> _PostorderStack; // az iterált változathoz egy vermet kell használni
        private List<TreeNode> _FoundNodes; // a már megtalált csúcsokat is egy listába helyezzük

        public override Boolean Running
        {
            get { return (_FoundNodes.Count > 0 || _PostorderStack.Count > 0); }
        }

        public PostorderTraversal(BinarySearchTree tree)
            : base(tree) // meghívjuk az ős konstruktorát
        {
            _PostorderStack = new Stack<TreeNode>();
            _FoundNodes = new List<TreeNode>();
        }

        public override void Start()
        {
            _PostorderStack.Clear();
            _FoundNodes.Clear();

            if (_Tree.Root != null) // ha a fa nem üres
                _PostorderStack.Push(_Tree.Root); // elkezdjük a bejárást a gyökérnél
        }

        public override void Step()
        {
            // a lépés során
            for (Int32 i = 0; i < _FoundNodes.Count; i++)
            {
                TreeNode node = _FoundNodes[i];
                // megnézzük, hogy az csúcsnak a gyerekeit (amennyiben vannak) már bejártuk-e
                if ((node.Left == null || _VisitedNodes.Contains(node.Left)) &&
                    (node.Right == null || _VisitedNodes.Contains(node.Right)))
                {
                    // ha igen
                    _FoundNodes.RemoveAt(i); // akkor kivesszük a listából
                    i--; // mivel a lista ekkor visszaugrik eggyet, csökkentjük a ciklusszámlálót
                    _VisitedNodes.Add(node); // felvesszük a bejártak listájára
                }
            }
            if (_PostorderStack.Count > 0) // ha még nem értük az összes csúcsot
            {
                TreeNode node = _PostorderStack.Pop(); // kivesszük a tetőelemet

                // akkor vesszük előbb a jobb, majd a bal gyereket, és berakjuk a bejárandók vermébe
                if (node.Right != null)
                    _PostorderStack.Push(node.Right);
                if (node.Left != null)
                    _PostorderStack.Push(node.Left);

                _FoundNodes.Add(node); // a megtaláltak vermébe helyezzük
            }
        }

        public override void Draw(Graphics gr, Int32 width)
        {
            if (_PostorderStack.Count > 0 || _FoundNodes.Count > 0 || _VisitedNodes.Count > 0) // ha már dolgozott az eljárás
                DrawNode(_Tree.Root, gr, 0, width, 20); // a saját rajzolófüggvényeket hívjuk
            else
                _Tree.Draw(gr, width); // különben a fát rajzoljuk ki
        }

        private void DrawNode(TreeNode node, Graphics gr, Int32 start, Int32 width, Int32 height)
        {
            Font font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold);
            Pen greenPen = new Pen(Color.Green);
            Pen redPen = new Pen(Color.Red);
            Pen bluePen = new Pen(Color.Blue); 
            Pen blackPen = new Pen(Color.Black);
            SizeF keySize = gr.MeasureString(node.Key.ToString(), font);

            if (node.Left != null)
            {
                gr.DrawLine(blackPen, start + width / 2, height, start + width / 4, height + 50);
                DrawNode(node.Left, gr, start, width / 2, height + 50);
            }
            if (node.Right != null)
            {
                gr.DrawLine(blackPen, start + width / 2, height, start + 3 * width / 4, height + 50);
                DrawNode(node.Right, gr, start + width / 2, width / 2, height + 50);
            }

            gr.FillEllipse(Brushes.White, start + width / 2 - 15, height - 15, 30, 30);
            if (_PostorderStack.Contains(node))
                gr.DrawEllipse(redPen, start + width / 2 - 15, height - 15, 30, 30);
            else if (_FoundNodes.Contains(node))
                gr.DrawEllipse(bluePen, start + width / 2 - 15, height - 15, 30, 30);
            else if (_VisitedNodes.Contains(node))
                gr.DrawEllipse(greenPen, start + width / 2 - 15, height - 15, 30, 30);
            else
                gr.DrawEllipse(blackPen, start + width / 2 - 15, height - 15, 30, 30);
            gr.DrawString(node.Key.ToString(), font, Brushes.Black, start + width / 2 - keySize.Width / 2, height - keySize.Height / 2); // belerajzoljuk a kulcsot
        }
    }
}
