﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Traversal.Model
{
    /// <summary>
    /// Preorder bejárás típusa.
    /// </summary>
    public class PreorderTraversal : TreeTraversal // a fa bejárás absztakt osztálynak egy leszármazottja
    {
        private Stack<TreeNode> _PreorderStack; // az iterált változathoz egy vermet kell használni

        public override Boolean Running
        {
            get { return (_PreorderStack.Count > 0); } // akkor fut, ha a verem nem üres
        }

        public PreorderTraversal(BinarySearchTree tree)
            : base(tree) // meghívjuk az ős konstruktorát
        {
            _PreorderStack = new Stack<TreeNode>();
        }

        public override void Start()
        {
            _PreorderStack.Clear(); // kezdéshez, ha már volt elem a sorban, akkor kiürítjük

            if (_Tree.Root != null) // ha nem üres a fa
                _PreorderStack.Push(_Tree.Root); // akkor felvesszük a gyökérelemet a verembe
        }

        public override void Step()
        {
            TreeNode node = _PreorderStack.Pop(); // a lépés során kivesszük a verem tetőelemét

            if (node.Right != null) // megnézzük van-e jobbgyereke
                _PreorderStack.Push(node.Right); // ha van, berakjuk a verembe
            if (node.Left != null) // ugyanez bal gyerekre
                _PreorderStack.Push(node.Left);
            _VisitedNodes.Add(node); // végül felvesszük a csúcsot a már bejártak közé
        }

        public override void Draw(Graphics gr, Int32 width)
        {
            if (_PreorderStack.Count > 0 || _VisitedNodes.Count > 0) // ha már dolgozott az eljárás
                DrawNode(_Tree.Root, gr, 0, width, 20); // a saját rajzolófüggvényeket hívjuk
            else
                _Tree.Draw(gr, width); // különben a fát rajzoljuk ki
        }

        private void DrawNode(TreeNode node, Graphics gr, Int32 start, Int32 width, Int32 height)
        {
            Font font = new Font(FontFamily.GenericSansSerif, 8, FontStyle.Bold);
            Pen greenPen = new Pen(Color.Green);
            Pen redPen = new Pen(Color.Red);
            Pen blackPen = new Pen(Color.Black);
            SizeF keySize = gr.MeasureString(node.Key.ToString(), font);

            if (node.Left != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + width / 4, height + 50);
                DrawNode(node.Left, gr, start, width / 2, height + 50);
            }
            if (node.Right != null)
            {
                gr.DrawLine(greenPen, start + width / 2, height, start + 3 * width / 4, height + 50);
                DrawNode(node.Right, gr, start + width / 2, width / 2, height + 50);
            }

            gr.FillEllipse(Brushes.White, start + width / 2 - 15, height - 15, 30, 30);
            if (_PreorderStack.Contains(node))
                gr.DrawEllipse(redPen, start + width / 2 - 15, height - 15, 30, 30);
            else if (_VisitedNodes.Contains(node))
                gr.DrawEllipse(greenPen, start + width / 2 - 15, height - 15, 30, 30);
            else
                gr.DrawEllipse(blackPen, start + width / 2 - 15, height - 15, 30, 30);
            gr.DrawString(node.Key.ToString(), font, Brushes.Black, start + width / 2 - keySize.Width / 2, height - keySize.Height / 2); // belerajzoljuk a kulcsot
        }
    }
}
