﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Traversal.Model
{
    /// <summary>
    /// Általános bináris keresőfa bejárás típusa.
    /// </summary>
    public abstract class TreeTraversal
    {
        protected BinarySearchTree _Tree; // a bejárandó fa
        protected List<TreeNode> _VisitedNodes; // a már bejárt csúcsok listája

        /// <summary>
        /// Algoritmus futásának lekérdezése.
        /// </summary>
        public abstract Boolean Running { get; } // a futás állapotának lekérdezése absztrakt, a konkrét bejárásban fogjuk megadni
        /// <summary>
        /// Az aktuális érték lekérdezése.
        /// </summary>
        public Int32 CurrentValue
        {
            get
            {
                if (_VisitedNodes.Count > 0) // ha már van bejárt csúcsunk a listában
                    return _VisitedNodes[_VisitedNodes.Count - 1].Key; // adjuk vissza a kulcs részét
                else
                    return Int32.MinValue; // különben adjunk vissza egy extremális értéket
            }
        }

        public TreeTraversal(BinarySearchTree tree) // konstruktor, direktben sosem hívjuk meg, csak a leszármazott osztályban
        {
            _Tree = tree; // megkapja, hogy melyik fát kell bejárnia
            _VisitedNodes = new List<TreeNode>(); // létrehozzuk a listát a meglátogatott elemeknek
        }
        /// <summary>
        /// Indítás.
        /// </summary>
        public abstract void Start(); // az indítás művelete absztrakt, a konkrét bejárásban fogjuk megadni
        /// <summary>
        /// Léptetés.
        /// </summary>
        public abstract void Step(); // a lépés művelete absztrakt, a konkrét bejárásban fogjuk megadni
        /// <summary>
        /// Teljes futtatás.
        /// </summary>
        /// <returns>A bejárt csúcsok listája.</returns>
        public List<Int32> Run()
        {
            List<Int32> values = new List<Int32>(); // a bejárt csúcsokat egy listába helyezzük el
            if (_Tree != null) // ha nem üres a fa
            {
                Start(); // indítsuk el az algoritmust
                while (Running) // amíg fut
                {
                    Step(); // lépjünk
                    if (CurrentValue > Int32.MinValue) // ha nem extremális értéken vagyunk, akkor van bejárt elem
                        values.Add(CurrentValue); // felvesszük egy listába
                }
                // at absztrakt műveletek helyett az az eljárás fog lefutni, amelyet a konkrét bejárásban megírunk
            }
            return values;
        }

        /// <summary>
        /// Algoritmus futásának kirajzolása.
        /// </summary>
        /// <param name="gr">Rajzobjektum, amivel rajzolunk.</param>
        /// <param name="width">A szélesség, amelyben rajzolunk.</param>
        public abstract void Draw(Graphics gr, Int32 width);
    }
}
