﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ELTE.Algorithms.BinarySearchTree.Traversal.Model;

namespace ELTE.Algorithms.BinarySearchTree.Traversal.View
{
    public partial class TreeForm : Form
    {
        private BinarySearchTree.Traversal.Model.BinarySearchTree _Tree; // mivel a típus neve megegyezik a névtérrel, a teljes hivatkozást le kell írnunk
        private TreeTraversal _Traversal;

        public TreeForm()
        {
            InitializeComponent();

            _Tree = new BinarySearchTree.Traversal.Model.BinarySearchTree(); // új bináris keresőfa
            _ComboSelectTraversal.SelectedIndex = 0;
        }

        private void ButtonSearch_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Search(Convert.ToInt32(_NumericKey.Value)); // művelet lefuttatása és eredmény lekérdezése
            _Panel.Refresh(); // panel frissítése
            if (result == null) // ha az eredmény null
                _StatusLabel.Text = "Key not found!";
            else
                _StatusLabel.Text = "Key found (marked red).";
        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Insert(Convert.ToInt32(_NumericKey.Value));
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Insert failed, key already in tree!";
            else
                _StatusLabel.Text = "Insert succeded.";
        }

        private void ButtonRemove_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Delete(Convert.ToInt32(_NumericKey.Value));
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Key not found, can't delete!";
            else
                _StatusLabel.Text = "Delete succeded.";
        }

        private void ButtonMin_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Min();
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Tree is empty!";
            else
                _StatusLabel.Text = "Minimum key is: " + result.Key.ToString();
        }

        private void ButtonMax_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Max();
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Tree is empty!";
            else
                _StatusLabel.Text = "Maximum key is: " + result.Key.ToString();
        }

        private void ButtonPrev_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Previous();
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Tree is empty!";
            else
                _StatusLabel.Text = "Previous key is: " + result.Key.ToString();
        }

        private void ButtonNext_Click(object sender, EventArgs e)
        {
            BinarySearchTree.Traversal.Model.TreeNode result = _Tree.Next();
            _Panel.Refresh();
            if (result == null)
                _StatusLabel.Text = "Tree is empty!";
            else
                _StatusLabel.Text = "Next key is: " + result.Key.ToString();
        }

        private void ButtonClear_Click(object sender, EventArgs e)
        {
            _Tree.Clear();
            _Panel.Refresh();
            _StatusLabel.Text = "Tree cleared.";
        }
        private void TreePanel_Paint(object sender, PaintEventArgs e) // panel frissítésének eseménykezelője
        {
            // automatikusan fehérre festi a hátteret, így ezzel nem kell foglalkoznunk
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias; // élsimítás beállítása
            if (_Traversal == null) // ha nincs még bejáró algoritmusunk
                _Tree.Draw(e.Graphics, _Panel.Width); // fa kirajzolása
            else
                _Traversal.Draw(e.Graphics, _Panel.Width); // algoritmus kirajzolása
        }

        private void ButtonStartStep_Click(object sender, EventArgs e)
        {
            // indítás, vagy léptetés
            if (_ButtonStartStep.Text == "Start") // ha a start felirat van a gombon
            {
                _ComboSelectTraversal.Enabled = false; // kikapcsoljuk a comboboxot
                switch (_ComboSelectTraversal.SelectedIndex) // megnézzük, hányadik sor van kiválasztva a comboboxban 
                {
                    case 0: // ha az 1. sor
                        _Traversal = new LevelOrderTraversal(_Tree); // akkor a bejárás szintfolytonos lesz
                        break;
                    case 1:
                        _Traversal = new PreorderTraversal(_Tree);
                        break;
                    case 2:
                        _Traversal = new PostorderTraversal(_Tree);
                        break;
                }
                if (_Traversal != null) // ha sikerült beállítani valami bejárást
                {
                    _Traversal.Start(); // akkor indítsuk
                    if (_Traversal.Running) // ha fut
                        _ButtonStartStep.Text = "Step"; // akkor módosítjuk a gomb szövegét
                }
            }
            else
            {
                _Traversal.Step(); // léptetjük az algoritmust
                if (_Traversal.CurrentValue > Int32.MinValue) // ha nem extremális értéket kaptunk aktuálisan bejárt elemként
                    _StatusLabel.Text = "Key " + _Traversal.CurrentValue.ToString() + " traversed.";
                if (!_Traversal.Running) // ha vége az eljárásnak
                {
                    _ButtonStartStep.Text = "Start"; // visszaállítjuk a feliratot
                    _ComboSelectTraversal.Enabled = true; // visszakapcsoljuk a comboboxot
                }
            }
            _Panel.Refresh();
        }

        private void ButtonRun_Click(object sender, EventArgs e)
        {
            // a teljes algoritmus futtatásánál egy Stringbe fűzzük össze a bejárt értékeket 
            String text = "Keys ";
            switch (_ComboSelectTraversal.SelectedIndex)
            {
                case 0:
                    _Traversal = new LevelOrderTraversal(_Tree);
                    break;
                case 1:
                    _Traversal = new PreorderTraversal(_Tree);
                    break;
                case 2:
                    _Traversal = new PostorderTraversal(_Tree);
                    break;
            }
            if (_Traversal != null)
            {
                List<Int32> values = _Traversal.Run(); // futtatjuk az eljárást
                foreach (Int32 val in values) // az eredmény listát feldolgozzuk elemenként
                    text += val.ToString() + " ";
                text += " traversed.";

                _StatusLabel.Text = text;
            }
            _ComboSelectTraversal.Enabled = true;
            _Panel.Refresh();
        }
    }
}
